//  This file is part of the computer game Kartofel.
//  Copyright (C) 2008 Pawel Aleksander Fedorynski <pfedor@fuw.edu.pl>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <cerrno>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <fcntl.h>
#include <iostream>
#include <soci.h>
#include <soci-mysql.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>

#include "config.h"
#include "game.h"
#include "saved_game.h"

const int NUM_LEVELS = 21;

using namespace std;
using namespace kartofel;
using namespace soci;

int main(int argc, char** argv)
{
  if (argc < 2) {
    cerr << "Usage: verifydb <connect-string>\n";
    exit(0);
  }
  session sql(mysql, argv[1]);

  vector<Level> levels;
  for (int i = 1; i <= NUM_LEVELS; i++) {
    char filename[1000];
    snprintf(filename, sizeof(filename), "levels/%d.txt", i);
    Level level;
    LoadLevel(filename, &level);
    levels.push_back(level);
  }

  for (;;) {
    int id, score;
    string game;
    sql << "select id, game, score from games where status = 'unknown' "
        << "and version like '" << VERSION << "%' "
        << "order by score desc limit 1", into(id), into(game), into(score);
    if (sql.got_data()) {
      // Save the game into a file.
      char filename[1000];
      snprintf(filename, sizeof(filename), "games/%d.game", id);
      FILE* f = fopen(filename, "w");
      if (!f) {
        cerr << "fopen(): " << filename << ": "
             << strerror(errno) << endl;
        exit(1);
      }
      if (game.size() < fwrite(game.data(), game.size(), 1, f)) {
        cerr << "fwrite(): " << filename << ": "
             << strerror(errno) << endl;
        exit(1);
      }
      if (0 != fclose(f)) {
        cerr << "fclose(): " << filename
             << ": " << strerror(errno) << endl;
      }

      // Verify the score.
      SavedGame sg;
      string new_status = "bad";
      if (sg.Decode(game)) {
        int actual_score, starting_level, end_level;
        if (sg.Verify(levels, &starting_level, &end_level, &actual_score)) {
          if (score == actual_score) {
            cout << "OK " << id << ": Level " << starting_level << ".."
                 << end_level << ", Score: " << score << endl;
            new_status = "verified";
          } else {
            cout << "WRONG " << id << ": Level " << starting_level << ".."
                 << end_level << ", Score: " << score << endl;
          }
        } else {
          cout << "WRONG " << id << ": Verify() failed.\n";
        }
      } else {
        cout << "WRONG " << id << ": Decode() failed.\n";
      }
      sql << "update games set status = :status where id = :id",
        use(new_status), use(id);
    } else {
      cout << "No games with status 'unknown'.\n";
    }
    sleep(60);
  }
}
